/* -----------------------------------------------------------------------------

 register.api 1999 Dietmar Eilert

 Plug-in example. This plug-in shows a button stripe, one button for each text.

 DMAKE

 -------------------------------------------------------------------------------

*/

#include "defs.h"

/// "prototypes"

// library functions

Prototype LibCall struct APIClient *APIMountClient(__A0 struct APIMessage *, __A1 char *);
Prototype LibCall void              APICloseClient(__A0 struct APIClient  *, __A1 struct APIMessage *);
Prototype LibCall void              APIBriefClient(__A0 struct APIClient  *, __A1 struct APIMessage *);
Prototype LibCall void              APIFree       (__A0 struct APIClient  *, __A1 struct APIOrder   *);

// private functions

Prototype UWORD  AllocateLayout   (struct PlugInContext *, UWORD);
Prototype void   ClearContainer   (struct PlugInContext *, struct APIMessage *);
Prototype ULONG  DispatchIE       (struct PlugInContext *, struct APIMessage *);
Prototype ULONG  DispatchContainer(struct PlugInContext *, struct APIMessage *);
Prototype void   FreeLayout       (struct PlugInContext *);
Prototype void   LayoutStripe     (struct PlugInContext *, struct APIMessage *);
Prototype void   RenderButton     (struct PlugInContext *, struct APIMessage *, struct APIContainer *, UWORD);
Prototype void   RenderStripe     (struct PlugInContext *, struct APIMessage *, BOOL);
Prototype UBYTE *TackOn           (UBYTE *, UBYTE *);
Prototype void   SendRexxAsync    (UBYTE *, UBYTE *);
Prototype UBYTE *mysprintf        (UBYTE *, UBYTE *, ...);

Prototype __stkargs UBYTE *myrawdofmt(UBYTE *, UBYTE *, APTR);

///
/// "global data"

struct AppContext AppContext = { 0 };

///
/// "library"

LibCall struct APIClient *
APIMountClient(__A0 struct APIMessage *apiMsg, __A1 char *args)
{
    struct PlugInContext *context;

    // build a description of this client (host will pass this back to us as "handle" during subsequent library calls)

    if (context = (struct PlugInContext *)AllocVec(sizeof(struct PlugInContext), MEMF_ANY | MEMF_CLEAR)) {

        struct Screen *screen;
        UWORD          height;
        UWORD          alignment;

        height    = 20;
        alignment = API_ALIGN_BOTTOM;

        // parse startup arguments (TOP/S,NOFILL/S,NOLINE/S)

        if (args) {

            if (strstr(args, "TOP"))

                alignment = API_ALIGN_TOP;

            if (strstr(args, "NOLINE"))

                context->Separator = 0;
            else
                context->Separator = 3;

            if (strstr(args, "NOFILL") == NULL)

                context->Backfill = TRUE;
        }
        else {

            context->Backfill  = TRUE;
            context->Separator = 3;
        }

        // fine-tune container height request

        if (screen = apiMsg->api_Instance->api_Screen) {

            if (context->Font = OpenDiskFont(screen->Font)) {

                height = context->Font->tf_YSize + 2;

                // stripe height must be a multiple of 4 to look nice

                height = ((height + 3) / 4) * 4;

                // add separator line

                height += context->Separator;
            }
        }

        // prepare the container request

        context->APIArea.api_Width        = API_SIZE_UNDEFINED;
        context->APIArea.api_Height       = height;
        context->APIArea.api_UnitsX       = API_UNITS_PIXEL;
        context->APIArea.api_UnitsY       = API_UNITS_PIXEL;
        context->APIArea.api_Alignment    = alignment;
        context->APIArea.api_Style        = API_STYLE_NOSIZEBAR | API_STYLE_NODRAGBAR | API_STYLE_BORDERLESS;
        context->APIArea.api_IDCMP        = IDCMP_MOUSEBUTTONS;

        // client description

        context->APIClient.api_APIVersion = API_INTERFACE_VERSION;
        context->APIClient.api_Version    = 1;
        context->APIClient.api_Name       = "Register";
        context->APIClient.api_Info       = "Plug-in example";
        context->APIClient.api_Commands   = NULL;
        context->APIClient.api_Serial     = 0;
        context->APIClient.api_Classes    = API_CLASS_SYSTEM | API_CLASS_CONTAINER | API_CLASS_WORKSPACE;
        context->APIClient.api_Area       = &context->APIArea;
    }

    return((struct APIClient *)context);
}

LibCall void
APICloseClient(__A0 struct APIClient *handle, __A1 struct APIMessage *apiMsg)
{
    struct PlugInContext *context = (struct PlugInContext *)handle;

    // free ressources related to this client

    if (context->Font)

        CloseFont(context->Font);

    FreeLayout(context);

    FreeVec(context);
}

LibCall void
APIBriefClient(__A0 struct APIClient *handle, __A1 struct APIMessage *apiMsg)
{
    struct APIMessage *msg;

    // handle host's command notify

    for (msg = apiMsg; msg; msg = msg->api_Next) {

        if (msg->api_State == API_STATE_NOTIFY) {

            switch (msg->api_Class) {

                case API_CLASS_CONTAINER:

                    msg->api_Error = DispatchContainer((struct PlugInContext *)handle, msg);

                    break;

                case API_CLASS_WORKSPACE:

                    if (((struct PlugInContext *)handle)->Attached)

                        RenderStripe((struct PlugInContext *)handle, apiMsg, TRUE);

                    break;

                case API_CLASS_SYSTEM:

                    msg->api_Error = API_ERROR_OK;

                    break;

                default:

                    msg->api_Error = API_ERROR_UNKNOWN;
            }
        }
    }
}


LibCall void
APIFree(__A0 struct APIClient *handle, __A1 struct APIOrder *apiOrder)
{
    struct APIOrder *order, *next;

    // free data allocated for orders sent to host

    for (order = apiOrder; order; order = next) {

        next = order->api_Next;

        FreeVec(order);
    }
}

///
/// "dispatcher"

/* ----------------------------- DispatchContainer -----------------------------

 Handle container-related event

*/

ULONG
DispatchContainer(context, apiMsg)

struct PlugInContext *context;
struct APIMessage    *apiMsg;
{
    ULONG error = API_ERROR_OK;

    switch (apiMsg->api_Action) {

        case API_ACTION_DETACH:

            // request to detach gadgets from window (we don't have gadgets)

            context->Attached = FALSE;

            break;

        case API_ACTION_ATTACH:

            // request to attach gadgets

            context->Attached = TRUE;

            LayoutStripe(context, apiMsg);

            break;

        case API_ACTION_PAINT:
        case API_ACTION_REFRESH:

            RenderStripe(context, apiMsg, FALSE);

            break;

        case API_ACTION_INPUTEVENT:

            error = DispatchIE(context, apiMsg);

            break;

        default:

            error = API_ERROR_UNKNOWN;
    }

    return(error);
}

///
/// "layout"

/* ------------------------------- LayoutStripe --------------------------------

 Layout the stripe

*/

void
LayoutStripe(context, apiMsg)

struct PlugInContext *context;
struct APIMessage    *apiMsg;
{
    struct APIContainer *container;

    context->Buttons = 0;

    if (container = apiMsg->api_Instance->api_Container) {

        if (context->Font == NULL)

            context->Font = OpenDiskFont(container->api_TextAttr);

        if (context->Font) {

            struct EditConfig *root;
            struct EditConfig *config;
            UWORD              buffers;

            // find first buffer

            root = (struct EditConfig *)apiMsg->api_Instance->api_Environment;

            while (root->Node.ln_Pred->ln_Pred)

                root = (struct EditConfig *)root->Node.ln_Pred;

            // count buffers/buttons

            buffers = 0;

            for (config = root; config->Node.ln_Succ; config = (struct EditConfig *)config->Node.ln_Succ)

                ++buffers;

            if (AllocateLayout(context, buffers)) {

                struct Rectangle *clipping;
                UWORD             height;

                context->Buttons = buffers;

                // container not fully obscured ?

                if (clipping = container->api_Clipping) {

                    height = context->Font->tf_YSize + 2;

                    // stripe height must be a multiple of 4 to look nice

                    height = ((height + 3) / 4) * 4;

                    // add separator line

                    height += context->Separator;

                    // container height sufficient ?

                    if (height <= (clipping->MaxY - clipping->MinY + 1)) {

                        context->MinX = clipping->MinX;
                        context->MinY = clipping->MinY;
                        context->MaxX = clipping->MaxX;
                        context->MaxY = clipping->MinY + height - 1;
                    }
                    else
                        height = 0;
                }
                else
                    height = 0;

                if (context->Height = height) {

                    struct ButtonContext *next;
                    struct TextFont      *font;
                    UWORD                 x;
                    UWORD                 y;

                    x = context->MinX;
                    y = context->MinY;

                    // consider reserved space for separator line above buttons

                    y      += context->Separator;
                    height -= context->Separator;

                    // the font is set in the layout phase to determine the raster size of strings

                    font = context->Font;

                    SetFont(container->api_RPort, font);

                    // layout buttons

                    config = root;

                    for (next = context->ButtonContext; buffers--; config = (struct EditConfig *)config->Node.ln_Succ, ++next) {

                        UWORD labelsize;
                        UWORD rastersize;

                        strncpy(next->Label, config->Name, MAX_FILELEN - 1);
                        strncpy(next->Path,  config->Path, MAX_PATHLEN - 1);

                        TackOn(next->Path, config->Name);

                        // button width is based on the label size

                        labelsize = TextLength(container->api_RPort, next->Label, strlen(next->Label));

                        // is this the button associated with our window/text ?

                        if (config == (struct EditConfig *)apiMsg->api_Instance->api_Environment) {

                            next->State = BUTTON_STATE_SELECTED;

                            // add extra space for bold effect

                            labelsize += font->tf_BoldSmear;
                        }
                        else
                            next->State = BUTTON_STATE_NORMAL;

                        // it should be an even number to perfectly center the text

                        if (labelsize & 1)

                            ++labelsize;

                        // add size of left/right borders (60 degrees)

                        rastersize = labelsize + (height - 2);

                        // store button dimensions

                        next->X = x;
                        next->Y = y;
                        next->W = rastersize;
                        next->H = height;

                        // calculate text position

                        next->TextX = x + (rastersize - labelsize) / 2;

                        next->TextY = y + (height - font->tf_YSize) / 2 + font->tf_Baseline;

                        // button not visible ?

                        if ((x + rastersize - 1) > context->MaxX)

                            next->State |= BUTTON_STATE_HIDDEN;

                        // advance to next output position (buttons overlap)

                        x += (labelsize + height / 2 - 1);
                    }
                }
            }
        }
    }
}


/* ------------------------------ AllocateLayout -------------------------------

 Ensure that we have enough layout memory to store the configuration data of at
 least <buttons> buttons.

*/

UWORD
AllocateLayout(context, buttons)

struct PlugInContext *context;
UWORD                 buttons;
{
    if (context->ButtonContextSize < buttons) {

        FreeLayout(context);

        // allocate extra space for future buttons

        buttons += 5;

        if (context->ButtonContext = (struct ButtonContext *)AllocVec(buttons * sizeof(struct ButtonContext), MEMF_CLEAR | MEMF_PUBLIC))

            context->ButtonContextSize = buttons;
    }

    return(context->ButtonContextSize);
}



/* -------------------------------- FreeLayout ---------------------------------

 Free layout data

*/

void
FreeLayout(context)

struct PlugInContext *context;
{
    if (context->ButtonContext)

        FreeVec(context->ButtonContext);

    context->ButtonContext     = NULL;
    context->ButtonContextSize = 0;
    context->Buttons           = 0;
}

///
/// "render"

/* ------------------------------- RenderStripe --------------------------------

 Render the stripe

*/

void
RenderStripe(context, apiMsg, reset)

struct PlugInContext *context;
struct APIMessage    *apiMsg;
BOOL                  reset;
{
    // relayout stripe (because buffers have been added or closed) ?

    if (reset) {

        LayoutStripe(context, apiMsg);

        ClearContainer(context, apiMsg);
    }

    if (context->ButtonContext == NULL)

        LayoutStripe(context, apiMsg);

    if (context->ButtonContext && context->Height) {

        struct APIContainer *container;

        if (container = apiMsg->api_Instance->api_Container) {

            struct RastPort *rast = container->api_RPort;

            UWORD button;

            // render separator line

            if (context->Separator >= 3) {

                SetAPen(rast, container->api_DrawInfo->dri_Pens[SHADOWPEN]);

                Move(rast, context->MinX, context->MinY + 1);

                Draw(rast, context->MaxX, context->MinY + 1);
            }

            // render all buttons

            for (button = 0; button < context->Buttons; ++button)

                RenderButton(context, apiMsg, container, button);
        }
    }
}


/* ------------------------------- RenderButton --------------------------------

 Render a button

*/

void
RenderButton(context, apiMsg, container, ID)

struct PlugInContext *context;
struct APIMessage    *apiMsg;
struct APIContainer  *container;
UWORD                 ID;
{
    struct ButtonContext *button = context->ButtonContext + ID;

    // button visible ?

    if ((button->State & BUTTON_STATE_HIDDEN) == FALSE) {

        struct RastPort *rast;
        UWORD            prevState;
        UWORD            nextState;
        UWORD            w;
        UWORD            h;
        UWORD            dx;
        UWORD            x0, x1, x2, x3, x4, x5, x6, x7;
        UWORD            y0, y1, y2, y3, y4, y5, y6, y7;

        // get state of previous/next buttons

        if (ID > 0)
            prevState = (context->ButtonContext + ID - 1)->State;
        else
            prevState = BUTTON_STATE_INDETERMINED;

        if ((ID + 1) < context->Buttons)
            nextState = (context->ButtonContext + ID + 1)->State;
        else
            nextState = BUTTON_STATE_INDETERMINED;

        // determine geometry

        w  = button->W;
        h  = button->H;

        dx = h / 2 - 1;                              // width of left/right border area (60 degrees)

        x0 = button->X;                              // top left
        y0 = button->Y;

        x1 = x0 + dx / 2;                            // middle of left border
        y1 = y0 + dx;

        x2 = x0 + dx;                                // lower left
        y2 = y0 + h - 1;

        x3 = x0 + w - h / 2;                         // right left
        y3 = y2;

        x5 = x0 + w - 1;                             // upper right
        y5 = y0;

        x4 = x5 - dx / 2;                            // middle of right border
        y4 = y1;

        x6 = x3;                                     // intersection upper border/left corner of next button
        y6 = y0;

        x7 = x2;                                     // intersection upper border/right corner of prev button
        y7 = y0;

        // render button

        rast = container->api_RPort;

        // active button ?

        if (button->State & BUTTON_STATE_SELECTED) {

            if (context->Backfill) {

                UWORD x, y;

                SetAPen(rast, container->api_DrawInfo->dri_Pens[SHINEPEN]);

                x = x0;
                y = y0;

                --w;

                while (y < y2) {

                    Move(rast, x,     y);
                    Draw(rast, x + w, y);

                    ++x;
                    ++y;

                    w -= 2;

                    Move(rast, x,     y);
                    Draw(rast, x + w, y);

                    ++y;
                }
            }
            else
                SetSoftStyle(rast, FSF_BOLD, FSF_BOLD);

            SetAPen(rast, container->api_DrawInfo->dri_Pens[SHADOWPEN]);

            if (button->Label && *button->Label) {

                SetDrMd(rast, JAM1);

                Move(rast, button->TextX, button->TextY);

                Text(rast, button->Label, strlen(button->Label));

                SetDrMd(rast, JAM2);
            }

            Move(rast, x0, y0);
            Draw(rast, x2, y2);                      // left border
            Draw(rast, x3, y3);                      // lower border
            Draw(rast, x5, y5);                      // right border

            SetSoftStyle(rast, 0, FSF_BOLD);
        }
        else {

            SetAPen(rast, container->api_DrawInfo->dri_Pens[SHADOWPEN]);

            // render unselected button: behind previous (unless no previous) and in front of next (unless next is selected)

            if (nextState & BUTTON_STATE_SELECTED)
                Move(rast, x6, y6);
            else
                Move(rast, x5, y5);

            if (prevState == BUTTON_STATE_INDETERMINED) {

                // this is the first button

                Draw(rast, x0, y0);                  // upper border
            }
            else {

                // we are behind the previous button

                Draw(rast, x7, y7);                  // upper border

                Move(rast, x1, y1);
            }

            Draw(rast, x2, y2);                      // left border
            Draw(rast, x3, y3);                      // lower border

            if (nextState & BUTTON_STATE_SELECTED)
                Draw(rast, x4, y4);                  // right border
            else
                Draw(rast, x5, y5);                  // right border

            if (button->Label && *button->Label) {

                Move(rast, button->TextX, button->TextY);

                Text(rast, button->Label, strlen(button->Label));
            }
        }

        SetDrMd(rast, JAM2);
    }
}


/* -------------------------------- ClearContainer -----------------------------

 Clear output area

*/

void
ClearContainer(context, apiMsg)

struct PlugInContext *context;
struct APIMessage    *apiMsg;
{
    struct APIContainer *container;

    if (container = apiMsg->api_Instance->api_Container) {

        struct Rectangle *clipping;

        // container not fully obscured ?

        if (clipping = container->api_Clipping) {

            struct RastPort *rast = container->api_RPort;

            // clear stripe

            SetAPen(rast, 0);

            RectFill(rast, clipping->MinX, clipping->MinY, clipping->MaxX, clipping->MaxY);
        }
    }
}

///
/// "events"

/* ------------------------------- DispatchClick -------------------------------

 Process mouse click

*/

ULONG
DispatchIE(context, apiMsg)

struct PlugInContext *context;
struct APIMessage    *apiMsg;
{
    struct APIInputEvent *event;

    ULONG error = API_ERROR_OK;

    if (event = (struct APIInputEvent *)apiMsg->api_Data) {

        if (event->api_IDCMPClass == IDCMP_MOUSEBUTTONS) {

            // is this event meant for our window ?

            if (event->api_IDCMPWindow == apiMsg->api_Instance->api_Window) {

                if (event->api_IDCMPCode == SELECTDOWN) {

                    if (context->Height) {

                        UWORD x, y;

                        x = event->api_IDCMPX;
                        y = event->api_IDCMPY;

                        // click over stripe ?

                        if ((x >= context->MinX) && (x <= context->MaxX)) {

                            if ((y >= context->MinY) && (y <= context->MaxY)) {

                                struct ButtonContext *found;
                                struct ButtonContext *next;
                                UWORD                 buttons;
                                UBYTE                *host;

                                // choose host

                                host = apiMsg->api_Instance->api_Host->api_Rexx;

                                if (host == NULL)

                                    host = "GOLDED.1";

                                // find button under pointer (there may be two buttons due to overlapping)

                                next = context->ButtonContext;

                                found  = NULL;

                                for (buttons = context->Buttons; buttons; --buttons, ++next) {

                                    if (x >= next->X) {

                                        if (x < (next->X + next->W)) {

                                            found = next;

                                            // stop if there can not be another overlapping button at this (x|y)

                                            if (found->State & BUTTON_STATE_SELECTED)

                                                break;
                                        }
                                    }
                                }

                                // activate text associated with button

                                if (found) {

                                    UBYTE command[MAX_COMMANDLEN];

                                    mysprintf(command, "WINDOW USE=\"%s\"", found->Path);

                                    SendRexxAsync(host, command);
                                }

                                // no further processing of this input event required

                                apiMsg->api_State = API_STATE_CONSUMED;
                            }
                        }
                    }
                }
            }
        }
    }

    return(error);
}


///
/// "rexx"

/* -------------------------------- SendRexxAsync ------------------------------

 Send asynchronous rexx message.

*/

void
SendRexxAsync(host, cmd)

UBYTE *host;
UBYTE *cmd;
{
    struct MsgPort *port;

    Forbid();

    if (port = FindPort(host)) {

        struct RexxMsg *msg;

        if (msg = CreateRexxMsg(AppContext.MsgPort, NULL, NULL)) {

            if (msg->rm_Args[0] = CreateArgstring(cmd, strlen(cmd))) {

                msg->rm_Action = RXCOMM | RXFF_RESULT;

                PutMsg(port, &msg->rm_Node);

                ++AppContext.Outstanding;
            }
        }
    }

    Permit();
}

///
/// "misc"

/* ---------------------------------- TackOn -----------------------------------

 Complete <path> by appending <file> (insert "/" if needed). Writes to <path> !

*/

UBYTE *
TackOn(path, file)

UBYTE *path;
UBYTE *file;
{
    UBYTE *last = path;

    if (*last) {

        last = strchr(last, 0);

        if ((last[-1] != ':') && (last[-1] != '/'))

            *last++ = '/';
    }

    strcpy(last, file);

    return(path);
}


/* ----------------------------------- mysprintf -------------------------------

 sprintf replacement

*/

__stkargs UBYTE *
mysprintf(UBYTE *buffer, UBYTE *template, ...)
{
    myrawdofmt(buffer, template, (APTR)((ULONG)&template + sizeof(ULONG)));

    return(buffer);
}

///
