/* -----------------------------------------------------------------------------

 1999 Dietmar Eilert. All Rights Reserved.

 Dietmar Eilert
 Mies-v-d-Rohe-Str.31, 52074 Aachen, Germany
 Phone: +49-(0)179-5987061 German/English
 E-Mail: Dietmar.Eilert@post.rwth-aachen.de
 E-Mail: dietmar_eilert@yahoo.de (alternative address)
 WWW:    http://members.tripod.com/golded
 Mirror: http://members.xoom.com/golded

 Syntax parser example. This example highlights everything after "//". DICE:

 DMAKE

 -------------------------------------------------------------------------------

*/

#include "defs.h"

/// "definitions"

// we extend the ParserHandle structure for our private data

struct MyParserHandle {

    struct ParserHandle  ParserHandle;               // embedded parser handle
    struct SyntaxChunk  *SyntaxStack;                // parser output
};

// syntax elements understood by parser

enum SyntaxLevels {

    SYNTAX_STANDARD,
    SYNTAX_COMMENT,
    SYNTAX_MAXIMUM
};

// names of built-in syntax levels

UBYTE *LevelNames[] = { "Standard text", "Comment (C++)", NULL };

///
/// "prototypes"

// library functions

Prototype LibCall struct ParserData     *MountScanner(void);
Prototype LibCall struct ParserHandle   *StartScanner(__A0 struct GlobalConfig *, __A1 struct EditConfig *, __D0 struct SyntaxChunk *, __D1 struct SyntaxSetup *);
Prototype LibCall ULONG                  CloseScanner(__A0 struct ParserHandle *);
Prototype LibCall void                   FlushScanner(__A0 struct ParserHandle *);
Prototype LibCall void                   SetupScanner(__A0 struct GlobalConfig *);
Prototype LibCall struct RefreshRequest *BriefScanner(__A0 struct ParserHandle *, __A1 struct ScannerNotify *);
Prototype LibCall struct SyntaxChunk    *ParseLine   (__A0 struct ParserHandle *, __A1 struct LineNode *, __D0 ULONG);
Prototype LibCall void                   UnparseLines(__A0 struct LineNode *, __D0 ULONG);
Prototype LibCall void                   ParseSection(__A0 struct ParserHandle *, __A1 struct LineNode *, __D0 ULONG);

// private functions

Prototype struct SyntaxChunk *ParseString(UBYTE *, UWORD, struct ParserHandle *);

///
/// "library functions"

/* ------------------------------- MountScanner --------------------------------

 Called by the editor before first usage of a scanner. We return a description of
 our abilities.

*/

LibCall struct ParserData *
MountScanner()
{
    static UBYTE version[] = "$VER: Comments 3.0 (" __COMMODORE_DATE__ ")";

    static struct ParserData parserData;

    static UBYTE *example[] = {

        "/* Simple ANSI C example */",
        "                           ",
        "#include <defs.h>          ",
        "                           ",
        "int                        ",
        "main(int argc, char **argv)",
        "{                          ",
        "   // anybody at home ?    ",
        "                           ",
        "   puts(\"Hello world !\");",
        "}                          ",

        NULL
    };

    parserData.pd_Release    = SCANLIBVERSION;
    parserData.pd_Version    = 1;
    parserData.pd_Serial     = 0;
    parserData.pd_Info       = "Comments 3.0";
    parserData.pd_Example    = example;
    parserData.pd_Flags      = 0;
    parserData.pd_Properties = 0;

    return(&parserData);
}


/* ------------------------------- StartScanner --------------------------------

 Called by the editor after a new text buffer has been created. We return a
 parser handle (extended with our private data).

*/

LibCall struct ParserHandle *
StartScanner(__A0 struct GlobalConfig *globalConfigPtr, __A1 struct EditConfig *editConfigPtr, __D0 struct SyntaxChunk *syntaxStack, __D1 struct SyntaxSetup *syntaxSetup)
{
    struct MyParserHandle *handle;

    if (handle = (struct MyParserHandle *)AllocVec(sizeof(struct MyParserHandle), MEMF_PUBLIC | MEMF_CLEAR)) {

        handle->ParserHandle.ph_Levels   = SYNTAX_MAXIMUM;
        handle->ParserHandle.ph_Names    = LevelNames;
        handle->ParserHandle.ph_ColorsFG = NULL;
        handle->ParserHandle.ph_ColorsBG = NULL;
        handle->SyntaxStack              = syntaxStack;
    }

    return((struct ParserHandle *)handle);
}


/* ------------------------------- CloseScanner --------------------------------

 Called by the editor if a text buffer is about to be closed. Deallocate buffer
 specific 'global' data.

*/

LibCall ULONG
CloseScanner(__A0 struct ParserHandle *parserHandle)
{
    if (parserHandle)

        FreeVec(parserHandle);

    return(0);
}


/* ------------------------------- FlushScanner --------------------------------

 Called by the editor in low memory situations: we are supposed to free as much
 memory as possible. We don't use a syntax cache, so there is nothing to free.

*/

LibCall void
FlushScanner(__A0 struct ParserHandle *parserHandle)
{
    ;
}


/* ------------------------------- SetupScanner --------------------------------

 Called by the editor if the user wants to change the scanner's configuration.
 We do not support user configuration (parserData.pd_Flags: SCPRF_CONFIGWIN flag
 unset).

*/

LibCall void
SetupScanner(__A0 struct GlobalConfig *globalConfigPtr)
{
    ;
}


/* ------------------------------- BriefScanner --------------------------------

 Called to notify a context scanner if lines have been added, deleted or
 modified. We aren't a context scanner (parserData.pd_Flags: SCPRF_CONTEXT
 flag unset), so we won't ever have to request additional display requests:
 the editor's built-in refresh of damage regions is sufficient.

*/

LibCall struct RefreshRequest *
BriefScanner(__A0 struct ParserHandle *parserHandle, __A1 struct ScannerNotify *notify)
{
    return(NULL);
}


/* --------------------------------- ParseLine ---------------------------------

 Parse a line, build a syntax description

*/

LibCall struct SyntaxChunk *
ParseLine(__A0 struct ParserHandle *parserHandle, __A1 struct LineNode *lineNode, __D0 ULONG line)
{
    if (IS_FOLD(lineNode)) {

        return(NULL);
    }
    else if (lineNode->Len) {

        return(ParseString(lineNode->Text, lineNode->Len, parserHandle));
    }
    else
        return(NULL);
}

/* -------------------------------- UnparseLines -------------------------------

 Called by the editor if lines are to be deleted. We are supposed to free
 private data attached to the lines. This scanner doesn't attach private data to
 lines (context scanners usally do in order to implement a syntax cache), so
 there is nothing to be freed.

*/

LibCall void
UnparseLines(__A0  struct LineNode *lineNode, __D0 ULONG lines)
{
    ;
}

/* -------------------------------- ParseSection -------------------------------

 Called by the editor if lines are to be displayed. The scanner is encouraged to
 preparse the lines. This scanner doesn't attach private data to lines (context
 scanners usally do in order to implement a syntax cache), so there is nothing
 to do.

*/

LibCall void
ParseSection(__A0 struct ParserHandle *parserHandle, __A1 struct LineNode *lineNode, __D0 ULONG lines)
{
    ;
}

///
/// "private"

/* -------------------------------- ParseString --------------------------------

 Parse a string, build a syntax description. This is a simple example only:
 C++-Comments (// ....) are highlighted. Return NULL if there is nothing to
 highlight.

*/

struct SyntaxChunk *
ParseString(UBYTE *text, UWORD len, struct ParserHandle *parserHandle)
{
    if (len) {

        BOOL  inString;
        UWORD indent;

        // leading spaces and tabs have to be ignored (we ignore all non-ascii characters)

        for (indent = 0; len && (*text <= 32); ++indent) {

            ++text;
            --len;
        }

        // trailing white space has to be ignored

        while (len && (text[len - 1] <= 32))

            --len;

        // check whether line containes a C++comment

        for (inString = FALSE; len >= 2; ++text, ++indent, --len) {

            if (*text == 34) {

                inString = !inString;
            }
            else if (inString == FALSE) {

                if ((text[0] == '/') && (text[1] == '/')) {

                    struct SyntaxChunk *syntaxStack = ((struct MyParserHandle *)parserHandle)->SyntaxStack;

                    syntaxStack[0].sc_Level = SYNTAX_COMMENT;
                    syntaxStack[0].sc_Start = indent;
                    syntaxStack[0].sc_End   = indent + len - 1;

                    syntaxStack[1].sc_Start = FALSE;
                    syntaxStack[1].sc_End   = FALSE;
                    syntaxStack[1].sc_Level = FALSE;

                    return(syntaxStack);
                }
            }
        }
    }

    return(NULL);
}

///
